// Unreal Fighting Framework by Kevin Poretti

#pragma once

// FF includes
#include "FFInputBufferComponent.h"

// UE includes
#include "CoreMinimal.h"
#include "GameFramework/PlayerController.h"
#include "InputMappingContext.h"

#include "FFPlayerController.generated.h"

/**
 *	A class that collects player inputs, stores them in an input buffer, and sends a rolling window of
 *	unacknowledged inputs to a remote client or server for processing.
 */
UCLASS()
class UNREALFIGHTINGFRAMEWORK_API AFFPlayerController : public APlayerController
{
	GENERATED_BODY()

public:
	AFFPlayerController();

	/**
	 *	Sends all unacknowledged inputs to the remote
	 */
	virtual void SendInputsToRemote() const;

	// APlayerController interface
	virtual void SetupInputComponent() override;
	// End of APlayerController interface

protected:
    /** MappingContext */
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category = "UFF|Input", meta = (AllowPrivateAccess = "true"))
	TSoftObjectPtr<UInputMappingContext> DefaultInputMapping;

	/** Input Buffer component */
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "UFF|Input", meta = (AllowPrivateAccess = "true"))
	UFFInputBufferComponent* InputBuffer;

	int32 Inputs;

	TArray<int32> UnacknowledgedInputs;
};
